/*!
 * \file
 *
 * \brief
 *
 * \author   Thomas Lochmatter
 *
 * \note     Copyright (C) 2006 EPFL
 */

#include <math.h>
#include "goto.h"
#define PI 3.14159265358979
#define ACCELERATION_STEP 200 //amount by which the robot accelerates/decelerates at each step
#define MAX_SPEED 10000.0f
#define MIN_SPEED 800.0f

void goto_start(tGoto * tg, tTrackPosition * tr) {
	tg->tr = tr;
	tg->goalx = 0;
	tg->goaly = 0;
	tg->speed1internal = 0;
	tg->speed2internal = 0;
	tg->speed1 = 0;
	tg->speed2 = 0;
	tg->atgoal = 1;
	tg->closetogoal = 1;
	tg->veryclosetogoal = 1;
	goto_setmaxspeed(tg, MAX_SPEED);
}

void goto_setgoal(tGoto * tg, float x, float y) {
	tg->goalx = x;
	tg->goaly = y;
	tg->atgoal = 0;
	tg->closetogoal = 0;
	tg->veryclosetogoal = 0;
}

void goto_setmaxspeed(tGoto * tg, float maxspeed) {
	tg->maxspeedfactor = maxspeed / PI;
}

void goto_step(tGoto * tg) {
	float dx, dy;
	float distance, goalangle, alpha;
	float speedfactor;
	long speed1wish, speed2wish;
	int atmaxspeed = 0;

	// Do nothing if we are at goal
	if (tg->atgoal != 0) {
		return;
	}

	// Calculate new wish speeds
	dx = tg->goalx - tg->tr->x;
	dy = tg->goaly - tg->tr->y;
	distance = sqrt(dx * dx + dy * dy);
	goalangle = atan2(dy, dx);
	alpha = goalangle - tg->tr->theta;
	while (alpha > PI) {
		alpha -= 2 * PI;
	}
	while (alpha < -PI) {
		alpha += 2 * PI;
	}

	speedfactor = (distance + 0.02) * MAX_SPEED;
	//speedfactor = (distance+0.01) * 400;

	if (speedfactor > tg->maxspeedfactor) {
		speedfactor = tg->maxspeedfactor;
		atmaxspeed = 1;
	}

	//printf("dist %f - goalangle %f - alpha %f \n", distance, goalangle, alpha);
	//speed1wish = speedfactor*(PI-2*abs(alpha)+alpha)+0.5;
	//speed2wish = speedfactor*(PI-2*abs(alpha)-alpha)+0.5;

	speed1wish = speedfactor * (PI - 2 * abs(alpha) + alpha) + 0.5;
	speed2wish = speedfactor * (PI - 2 * abs(alpha) - alpha) + 0.5;

	// Close to termination condition: just stop
	if (tg->veryclosetogoal) {
		speed1wish = 0;
		speed2wish = 0;
	}

	// Limit acceleration
	if (speed1wish > tg->speed1internal) {
		tg->speed1internal += ACCELERATION_STEP;
	}
	if (speed1wish < tg->speed1internal) {
		tg->speed1internal -= ACCELERATION_STEP;
	}
	if (speed2wish > tg->speed2internal) {
		tg->speed2internal += ACCELERATION_STEP;
	}
	if (speed2wish < tg->speed2internal) {
		tg->speed2internal -= ACCELERATION_STEP;
	}

	// Don't set speeds < MIN_SPEED (for accuracy reasons)
	tg->speed1 = tg->speed1internal;
	tg->speed2 = tg->speed2internal;

	if (abs(tg->speed1) < MIN_SPEED) {
		tg->speed1 = 0;
	}
	if (abs(tg->speed2) < MIN_SPEED) {
		tg->speed2 = 0;
	}

	// Termination condition
	if (atmaxspeed == 0) {
		tg->closetogoal = 1;
		if ((tg->speed1 == 0) || (tg->speed2 == 0)) {
			tg->veryclosetogoal = 1;
		}
		if ((tg->speed1 == 0) && (tg->speed2 == 0)) {
			tg->atgoal = 1;
		}
	}
}
