/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include "i2cal.h"

struct i2c_msg i2cal_messages[I2C_MESSAGES_LEN];
int i2cal_messages_nw = 0;
#define WRITE_BUFFER_LEN 256
char write_buffer[WRITE_BUFFER_LEN];
int write_buffer_nw = 0;
int write_buffer_start = 0;
#define READ_BUFFER_LEN 256
char read_buffer[READ_BUFFER_LEN];
int read_buffer_nw = 0;

int i2cal_fd = -1;

int i2cal_init() {
}

int i2cal_open() {
	// If already open, do nothing
	if (i2cal_fd >= 0) {
		return 2;
	}

	// Try to open
	i2cal_fd = open("/dev/i2c/0", O_RDWR);
	if (i2cal_fd < 0) {
		return 0;
	}

	// If successful, initialize variables
	i2cal_start();
	return 1;
}

void i2cal_close() {
	if (i2cal_fd < 0) {
		return;
	}
	close(i2cal_fd);
	i2cal_fd = -1;
}

void i2cal_start() {
	// Initialize variables for the next transfer
	i2cal_messages_nw = 0;
	write_buffer_nw = 0;
	write_buffer_start = 0;
	read_buffer_nw = 0;
}

struct i2c_msg *i2cal_read_buffer(int dev, unsigned char *buffer, int len) {
	struct i2c_msg *msg;

	// If our message buffer is full, give up
	if (i2cal_messages_nw > I2C_MESSAGES_LEN) {
		return 0;
	}

	// Create and add a new read message
	msg = &(i2cal_messages[i2cal_messages_nw]);
	i2cal_messages_nw++;

	msg->addr  = dev;
	msg->flags = I2C_M_RD;
	msg->buf   = buffer;
	msg->len   = len;
	//read_buffer_nw+=len;

	return msg;
}

struct i2c_msg *i2cal_write_buffer(int dev, unsigned char *buffer, int len) {
	struct i2c_msg *msg;

	// If our message buffer is full, give up
	if (i2cal_messages_nw > I2C_MESSAGES_LEN) {
		return 0;
	}

	// Create and add a new write message
	msg = &(i2cal_messages[i2cal_messages_nw]);
	i2cal_messages_nw++;

	msg->addr  = dev;
	msg->flags = 0;  // TODO: add proper flag
	msg->buf   = buffer;
	msg->len   = len;
	//write_buffer_start=write_buffer_nw;

	return msg;
}

struct i2c_msg *i2cal_read(int dev, int len) {
	struct i2c_msg *msg;

	// If our read buffer would be full, give up
	if (read_buffer_nw + len > READ_BUFFER_LEN) {
		return 0;
	}

	// Add a new read message and reserve the necessary space in the read buffer
	msg = i2cal_read_buffer(dev, &(read_buffer[read_buffer_nw]), len);
	if (msg == 0) {
		return 0;
	}
	read_buffer_nw += len;

	return msg;
}

struct i2c_msg *i2cal_write(int dev) {
	struct i2c_msg *msg;

	// If our write buffer would be full, give up
	if (write_buffer_nw > WRITE_BUFFER_LEN) {
		return 0;
	}

	// Add a new write message with the content of the write buffer
	msg = i2cal_write_buffer(dev, &(write_buffer[write_buffer_start]), write_buffer_nw - write_buffer_start);
	if (msg == 0) {
		return 0;
	}
	write_buffer_start = write_buffer_nw;

	return msg;
}

unsigned char *i2cal_writedata_uint8(unsigned char value) {
	unsigned char *buf = i2cal_writedata_buffer(1);

	if (buf == 0) {
		return 0;
	}
	*buf = value;
	return buf;
}

unsigned char *i2cal_writedata_int16(int value) {
	unsigned char *arr = (unsigned char *)(&value);
	unsigned char *buf = i2cal_writedata_buffer(2);

	if (buf == 0) {
		return 0;
	}
	buf[0] = arr[0];
	buf[1] = arr[1];
	return buf;
}

unsigned char *i2cal_writedata_uint16(unsigned int value) {
	unsigned char *arr = (unsigned char *)(&value);
	unsigned char *buf = i2cal_writedata_buffer(2);

	if (buf == 0) {
		return 0;
	}
	buf[0] = arr[0];
	buf[1] = arr[1];
	return buf;
}

unsigned char *i2cal_writedata_int32(int value) {
	unsigned char *arr = (unsigned char *)(&value);
	unsigned char *buf = i2cal_writedata_buffer(4);

	if (buf == 0) {
		return 0;
	}
	buf[0] = arr[0];
	buf[1] = arr[1];
	buf[2] = arr[2];
	buf[3] = arr[3];
	return buf;
}

unsigned char *i2cal_writedata_uint32(unsigned int value) {
	unsigned char *arr = (unsigned char *)(&value);
	unsigned char *buf = i2cal_writedata_buffer(4);

	if (buf == 0) {
		return 0;
	}
	buf[0] = arr[0];
	buf[1] = arr[1];
	buf[2] = arr[2];
	buf[3] = arr[3];
	return buf;
}

unsigned char *i2cal_writedata_buffer(int len) {
	unsigned char *buf;

	// Reserve bytes in the write buffer (which can then be filled by other functions)
	if (write_buffer_nw + len > WRITE_BUFFER_LEN) {
		write_buffer_nw = WRITE_BUFFER_LEN + 1;
		return 0;
	}
	buf = &(write_buffer[write_buffer_nw]);
	write_buffer_nw += len;
	return buf;
}

int i2cal_commit() {
	struct i2c_rdwr_ioctl_data msgset;

	// Create a message set
	msgset.msgs = i2cal_messages;
	msgset.nmsgs = i2cal_messages_nw;

	// Execute that message set
	return ioctl(i2cal_fd, I2C_RDWR, &msgset);
}

unsigned char i2cal_readdata_uint8(struct i2c_msg *message, int offset) {
	return message->buf[offset];
}

int i2cal_readdata_int16(struct i2c_msg *message, int offset) {
	unsigned char arr[2];
	short int *value = (short int *)arr;

	arr[0] = message->buf[offset];
	arr[1] = message->buf[++offset];
	return (int)(*value);
}

unsigned int i2cal_readdata_uint16(struct i2c_msg *message, int offset) {
	unsigned char arr[4];
	unsigned int *value = (unsigned int *)arr;

	arr[0] = message->buf[offset];
	arr[1] = message->buf[++offset];
	arr[2] = 0;
	arr[3] = 0;
	return *value;
}

int i2cal_readdata_int32(struct i2c_msg *message, int offset) {
	unsigned char arr[4];
	int *value = (int *)arr;

	arr[0] = message->buf[offset];
	arr[1] = message->buf[++offset];
	arr[2] = message->buf[++offset];
	arr[3] = message->buf[++offset];
	return *value;
}

unsigned int i2cal_readdata_uint32(struct i2c_msg *message, int offset) {
	unsigned char arr[4];
	unsigned int *value = (unsigned int *)arr;

	arr[0] = message->buf[offset];
	arr[1] = message->buf[++offset];
	arr[2] = message->buf[++offset];
	arr[3] = message->buf[++offset];
	return *value;
}
