/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#ifndef KHEPERA3_BATTERY
#define KHEPERA3_BATTERY

#include "i2cal.h"

//! Battery
struct sKhepera3Battery {
	unsigned int voltage;						//!< Voltage (0.1 mV)
	int current;								//!< Instantaneous current (0.1 mA)
	int current_average;						//!< Average current (0.1 mA)
	unsigned int capacity_remaining_absolute;	//!< Absolute remaining capacity (0.1 mAh)
	unsigned int capacity_remaining_relative;	//!< Relative remaining capacity (0.0001 %)
	unsigned int temperature;					//!< Temperature (0.0001 degrees Celsius)
};

//! Initializes this module.
void khepera3_battery_init();

//! (private) Reads a piece of battery information. The return value indicates success (-1) or failure (0).
int khepera3_battery_p(int *result, char argument);

//! Reads the battery voltage and updates this value in the khepera3 structure. The return value indicates success (-1) or failure (0).
int khepera3_battery_voltage();
//! Reads the battery current and updates this value in the khepera3 structure. The return value indicates success (-1) or failure (0).
int khepera3_battery_current();
//! Reads the average current of the battery and updates this value in the khepera3 structure. The return value indicates success (-1) or failure (0).
int khepera3_battery_current_average();
//! Reads the absolute remaining capacity of the battery and updates this value in the khepera3 structure. The return value indicates success (-1) or failure (0).
int khepera3_battery_capacity_remaining_absolute();
//! Reads the relative remaining capacity of the battery and updates this value in the khepera3 structure. The return value indicates success (-1) or failure (0).
int khepera3_battery_capacity_remaining_relative();
//! Reads the battery temperature and updates this value in the khepera3 structure. The return value indicates success (-1) or failure (0).
int khepera3_battery_temperature();

#endif
