/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#ifndef KHEPERA3_ULTRASOUND
#define KHEPERA3_ULTRASOUND

#include "i2cal.h"

//! Ultrasound sensor names
enum eKhepera3SensorsUltrasound {
	cKhepera3SensorsUltrasound_Left = 0,
	cKhepera3SensorsUltrasound_FrontLeft = 1,
	cKhepera3SensorsUltrasound_Front = 2,
	cKhepera3SensorsUltrasound_FrontRight = 3,
	cKhepera3SensorsUltrasound_Right = 4
};

//! Ultrasound sensor name strings
const char *cKhepera3SensorsUltrasound[5];

//! One measurement of one ultrasound sensor
struct sKhepera3SensorsUltrasoundSensor {
	int echos_count;		//!< Number of received echos
	int distance[10];		//!< Received distance (TODO: what unit?)
	int amplitude[10];		//!< Received amplitude
	int timestamp;			//!< Timestamp at which the values were read
};

//! The ultrasound sensors
struct sKhepera3SensorsUltrasound {
	struct sKhepera3SensorsUltrasoundSensor sensor[5];	//!< The 5 ultrasound sensors
	float distance_per_increment;
};

//! Initializes this module.
void khepera3_ultrasound_init();

//! (private) Reads the last values of an ultrasound sensor and fills them into the provided structure. The return value indicates success (-1) or failure (0).
int khepera3_ultrasound_p(struct sKhepera3SensorsUltrasoundSensor *result, int sensor);

//! Reads the last values of an ultrasound sensor and fills them into the khepera3 structure. The return value indicates success (-1) or failure (0).
int khepera3_ultrasound(int sensor);
//! Enables or disables ultrasound sensors. The return value indicates success (-1) or failure (0).
int khepera3_ultrasound_enable(int bitmask);
//! Sets the maximum number of echos received by the ultrasound sensors. The return value indicates success (-1) or failure (0).
int khepera3_ultrasound_set_max_echo_number(int max_echo_number);
//! Returns a sensor ID by name.
int khepera3_ultrasound_getsensorbyname(const char *name, int defaultvalue);

#endif
