/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include <math.h>
#include "trackposition.h"
#include "khepera3.h"
#define PI 3.14159265358979

int trackposition_start(struct sTrackPosition * tp) {
	khepera3_drive_get_current_position();
	return trackposition_start_pos(tp, -khepera3.motor_left.current_position, khepera3.motor_right.current_position);
}

int trackposition_start_pos(struct sTrackPosition * tp, int pos_left, int pos_right) {
	FILE *file;
	char buffer[128];
	char *token;
	double stepspertour;

	tp->x = 0;
	tp->y = 0;
	tp->theta = 0;
	tp->pos_left_prev = pos_left;
	tp->pos_right_prev = pos_right;

	// Distance between the wheels and wheel radii: default values
	tp->wheel_distance = 0.0894;
	trackposition_setwheelconfiguration_left(tp, 0.02, 2764.6);
	trackposition_setwheelconfiguration_right(tp, 0.02, 2764.6);

	// Read from file if available
	file = fopen("/etc/khepera/odometry", "r");
	if (! file) {
		return 0;
	}
	if (! fgets(buffer, 128, file)) {
		return 0;
	}

	token = strtok(buffer, " \t,");
	if (token == NULL) {
		return 0;
	}
	stepspertour = atof(token);

	token = strtok(NULL, " \t,");
	if (token == NULL) {
		return 0;
	}
	trackposition_setwheelconfiguration_left(tp, atof(token), stepspertour);

	token = strtok(NULL, " \t,");
	if (token == NULL) {
		return 0;
	}
	stepspertour = atof(token);

	token = strtok(NULL, " \t,");
	if (token == NULL) {
		return 0;
	}
	trackposition_setwheelconfiguration_right(tp, atof(token), stepspertour);

	token = strtok(NULL, " \t,");
	if (token == NULL) {
		return 0;
	}
	tp->wheel_distance = atof(token);

	fclose(file);
	return 1;
}

void trackposition_step(struct sTrackPosition * tp) {
	khepera3_drive_get_current_position();
	trackposition_step_pos(tp, -khepera3.motor_left.current_position, khepera3.motor_right.current_position);
}

void trackposition_step_pos(struct sTrackPosition * tp, int pos_left, int pos_right) {
	long delta_pos_left, delta_pos_right;
	float delta_left, delta_right, delta_theta, theta2;
	float delta_x, delta_y;

	delta_pos_left = pos_left - tp->pos_left_prev;
	delta_pos_right = pos_right - tp->pos_right_prev;
	delta_left = delta_pos_left * tp->wheel_conversion_left;
	delta_right = delta_pos_right * tp->wheel_conversion_right;
	delta_theta = (delta_left - delta_right) / tp->wheel_distance;
	theta2 = tp->theta + delta_theta * 0.5;
	delta_x = (delta_left + delta_right) * 0.5 * cosf(theta2);
	delta_y = (delta_left + delta_right) * 0.5 * sinf(theta2);

	tp->x += delta_x;
	tp->y += delta_y;
	tp->theta += delta_theta;
	tp->pos_left_prev = pos_left;
	tp->pos_right_prev = pos_right;
}

void trackposition_setwheelconfiguration_left(struct sTrackPosition * tp, float radius, float stepspertour) {
	tp->wheel_conversion_left = 2 * PI * radius / stepspertour;
}

void trackposition_setwheelconfiguration_right(struct sTrackPosition * tp, float radius, float stepspertour) {
	tp->wheel_conversion_right = 2 * PI * radius / stepspertour;
}
