/*! 
 * \file
 *
 * \brief
 *
 * \author   Thomas Lochmatter
 *
 * \note     Copyright (C) 2006 EPFL
 */

#include <korebot/korebot.h>
#include <math.h>
#include "trackposition.h"
#include "goto.h"

/*! handle to the various khepera3 devices (knet socket, i2c mode)
 */
//static knet_dev_t * dsPic;
static knet_dev_t * mot1;
static knet_dev_t * mot2;

// Current position
float curx, cury, curtheta;

/*--------------------------------------------------------------------*/
/*! Prints the help text.
 */
void help() {
	printf("Goes to a given position and a given angle\r\n");
	printf("Usage:\r\n");
	printf("  goto X Y [MAXSPEED]\r\n");
	printf("  goto FILE\r\n");
}

/*--------------------------------------------------------------------*/
/*! Main program.
 */
int run(int argc, char *argv[]) {
	tTrackPosition tr;
	tGoto tg;
	FILE *file;
	char buffer[128];
	char *token;
	float x, y;

	trackposition_start_mot(&tr, mot1, mot2);
	goto_start(&tg, &tr);

	if (argc>2) {
		// If two numbers are given, go to that position
		x=atof(argv[1]);
		y=atof(argv[2]);
		if (argc>3) {
		  goto_setmaxspeed(&tg, atof(argv[3]));
		}
		goto_setgoal(&tg, x, y);
		while (tg.atgoal==0) {
			runstep(&tg);
		}

	} else if (argc==2) {
		// If a filename is given, read positions from that file
		file=fopen(argv[1], "r");
		if (file==NULL) {
			printf("Unable to open file '%s' for reading\n", argv[1]);
			return 2;
		}

		// First line defines start position and track
		if (! fgets(buffer, 128, file)) {return;}

		token=strtok(buffer, " \t,");
		if (token==NULL) {return;}
		tg.tr->x = atof(token);

		token=strtok(NULL, " \t,");
		if (token==NULL) {return;}
		tg.tr->y = atof(token);

		token=strtok(NULL, " \t,");
		if (token) {
			tg.tr->theta = atof(token);
		}

		token=strtok(NULL, " \t,");
		if (token) {
			goto_setmaxspeed(&tg, atof(token));
		}

		// Go to each point (but only close to it)
		while (fgets(buffer, 128, file)) {
			token=strtok(buffer, " \t,");
			if (token==NULL) {break;}
			x = atof(token);

			token=strtok(NULL, " \t,");
			if (token==NULL) {break;}
			y = atof(token);

			goto_setgoal(&tg, x, y);
			while (tg.closetogoal==0) {
				runstep(&tg);
			}
		}

		// Go to the last point
		while (tg.atgoal==0) {
			runstep(&tg);
		}

		// Close file
		fclose(file);

	} else {
		help();
		return 1;
	}
	
	return 0;
}
 
int runstep(tGoto * tg) {
	// Update position and calculate new speeds
	trackposition_step_mot(tg->tr, mot1, mot2);
	goto_step(tg);

	// Set speed
	kmot_SetPoint(mot1, kMotRegSpeed, -tg->speed1);
	kmot_SetPoint(mot2, kMotRegSpeed, tg->speed2);

	printf("%d, %d, %f, %f, %f, %f, %f, %d, %d\r\n", tg->tr->pos1prev, tg->tr->pos2prev, tg->tr->x, tg->tr->y, tg->tr->theta, tg->goalx, tg->goaly, tg->speed1, tg->speed2);
	return 0;
}

/*--------------------------------------------------------------------*/
/*! Initialize and run the program.
 */
int main(int argc, char *argv[]) {
	
	// General initialization
	kh3_init();

	// Open dspic
	/*dsPic = knet_open( "Khepera3:dsPic" , KNET_BUS_I2C , 0 , NULL );
	if (dsPic==0) {
		printf("dsPIC not found\r\n");
		return 255;
	} */
	
	// Open motors
	mot1=knet_open("Khepera3:mot1", KNET_BUS_I2C, 0, NULL);
	mot2=knet_open("Khepera3:mot2", KNET_BUS_I2C, 0, NULL);
	if ((mot1==0) || (mot2==0)) {
		printf("Motors not found\r\n");
		return 255;
	}

	// Help
	if ((argc==2) && ((strcmp(argv[1], "help")==0) || (strcmp(argv[1], "-help")==0) || (strcmp(argv[1], "-h")==0) || (strcmp(argv[1], "--help")==0))) {
		help();
		return 0;
	}

	return run(argc, argv);
} 
