/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include "khepera3.h"
#include "commandline.h"
#include "measurement.h"

// Buffers
struct sMeasurement {
	int sample_number;
	struct sKhepera3SensorsInfrared infrared_ambient;
};

#define LOG_SIZE 1024
struct sMeasurement log[LOG_SIZE];
unsigned int sample_number = 0;

// Prints the help text.
void help() {
	printf("Measures the ambient infrared light with the infrared sensors.\r\n");
	printf("Usage:\r\n");
	printf("  infrared_ambient             Measures one set of values\r\n");
	printf("  infrared_ambient -r [N]      Measures N sets of values (N=infinity if not provided)\r\n");
	printf("  infrared_ambient -r N -f     Measures N sets of values and keeps them in memory before printing (for fast acquisition)\r\n");
	printf("  infrared_ambient -r N -F     Measures N sets of values but does not print them\r\n");
	printf("  infrared_ambient -w US       Waits US us after each measurement\r\n");
	printf("Output format:\r\n");
	printf("  $IRAMBIENT,sample,timestamp,value0,...,value10\r\n");
}

// Takes one measurement
void measurement_take(int i) {
	khepera3_infrared_ambient_p(&(log[i].infrared_ambient));
	log[i].sample_number = sample_number;
	sample_number++;
}

// Prints one measurement
void measurement_print(int i) {
	int s;
	printf("$IRAMBIENT,%d,%lu", log[i].sample_number, log[i].infrared_ambient.timestamp);
	for (s = 0; s < 11; s++) {
		printf(",%d", log[i].infrared_ambient.sensor[s]);
	}
	printf("\r\n");
}

// Main program.
int main(int argc, char *argv[]) {
	// Command line parsing
	commandline_init();
	measurement_init();
	measurement_commandline_prepare();
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Take continuous measures
	measurement_configuration.log_size = LOG_SIZE;
	measurement_configuration.hook_measure = &measurement_take;
	measurement_configuration.hook_print = &measurement_print;
	measurement_start();

	return 0;
}

