/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include "khepera3.h"
#include "commandline.h"

// Algorithm parameters and results
struct sForwardBackward {
	int speed;
	long int move_us;
	long int stop_us;
};

// Level of verbosity (0=quiet, 1=normal, 2=verbose)
int verbosity = 1;

// Prints the help text.
void help() {
	printf("Alternately moves forwards and backwards.\r\n");
	printf("Usage: motion_forwardbackward [OPTIONS]\r\n");
	printf("  --speed SPEED    Sets the speed (default: 10000)\r\n");
	printf("  --move-us US     Sets the time the robot moves back or forth (default: 2000000)\r\n");
	printf("  --stop-us US     Sets the time the robot waits when changing direction (default: 0)\r\n");
	printf("  -q --quiet       Be quiet (no output)\r\n");
	printf("  -v --verbose     Be verbose\r\n");
	printf("Output:\r\n");
	printf("  $DIRECTION,[FORWARD|BACKWARD|STOP]\r\n");
}

void forwardbackward_init(struct sForwardBackward *forwardbackward) {
	forwardbackward->speed = 10000;
	forwardbackward->move_us = 2000*1000;
	forwardbackward->stop_us = 0;
}

// Follows a line.
void forwardbackward_run(struct sForwardBackward *forwardbackward) {
	// Put the wheels in normal (control) mode
	khepera3_drive_start();

	while (1) {
		// Move forward
		if (verbosity > 0) {
			printf("$DIRECTION,FORWARD\r\n");
		}
		khepera3_drive_set_speed(forwardbackward->speed, forwardbackward->speed);
		usleep(forwardbackward->move_us);

		// Stop
		if (forwardbackward->stop_us > 0) {
			if (verbosity > 0) {
				printf("$DIRECTION,STOP\r\n");
			}
			khepera3_drive_set_speed(0, 0);
			usleep(forwardbackward->stop_us);
		}

		// Move backward
		if (verbosity > 0) {
			printf("$DIRECTION,BACKWARD\r\n");
		}
		khepera3_drive_set_speed(-forwardbackward->speed, -forwardbackward->speed);
		usleep(forwardbackward->move_us);

		// Stop
		if (forwardbackward->stop_us > 0) {
			if (verbosity > 0) {
				printf("$DIRECTION,STOP\r\n");
			}
			khepera3_drive_set_speed(0, 0);
			usleep(forwardbackward->stop_us);
		}
	}
}

// Main program.
int main(int argc, char *argv[]) {
	struct sForwardBackward forwardbackward;
	struct sCommandLineOption *option_speed;
	struct sCommandLineOption *option_move_us;
	struct sCommandLineOption *option_stop_us;

	option_speed = commandline_option_register("-s", "--speed", cCommandLine_Option_Value);
	option_move_us = commandline_option_register("-m", "--move-us", cCommandLine_Option_Value);
	option_stop_us = commandline_option_register("-p", "--stop-us", cCommandLine_Option_Value);

	// Command line parsing
	commandline_init();
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();
	forwardbackward_init(&forwardbackward);

	// Read command line arguments
	if (option_speed->value) {
		forwardbackward.speed = atoi(option_speed->value);
	}
	if (option_move_us->value) {
		forwardbackward.move_us = atoi(option_move_us->value);
	}
	if (option_stop_us->value) {
		forwardbackward.stop_us = atoi(option_stop_us->value);
	}
	if (commandline_option_provided("-q", "--quiet")) {
		verbosity = 0;
	}
	if (commandline_option_provided("-v", "--verbose")) {
		verbosity = 2;
	}

	// Run the algorithm
	forwardbackward_run(&forwardbackward);
	return 0;
}
