/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include "khepera3.h"
#include "commandline.h"
#include <stdlib.h>

// Algorithm parameters
struct sMotionRandom {
	int speed;
	float deviation_max;
	long int interval_us;
};

// Level of verbosity (0=quiet, 1=normal, 2=verbose)
int verbosity = 1;


void help() {
	printf("Moves the robot randomly around.\r\n");
	printf("Usage: motion_random [OPTIONS]\r\n");
	printf("  -s --speed SPEED      Sets the forward speed (default: 10000)\r\n");
	printf("  -d --deviation DEV    Sets the maximum deviation (default: 10000)\r\n");
	printf("  -i --interval-us US   Sets the interval between speed changes (default: 1000000)\r\n");
}

int motion_random_init(struct sMotionRandom *motion_random) {
	motion_random->speed=10000;
	motion_random->deviation_max=10000;
	motion_random->interval_us=1000*1000;
}

int motion_random_run(struct sMotionRandom *motion_random) {
	int speed_left, speed_right;
	double direction = 0;

	// Put the wheels in normal (control) mode
	khepera3_drive_start();

	while (1) {
		direction=((double)rand()/(double)RAND_MAX)*motion_random->deviation_max*2-motion_random->deviation_max;
		speed_left=(int)floorf(motion_random->speed+direction+0.5);
		speed_right=(int)floorf(motion_random->speed-direction+0.5);
		printf("$SPEED,%d,%d\r\n", speed_left, speed_right);
		khepera3_drive_set_speed(speed_left, speed_right);

		usleep(motion_random->interval_us);
	}
	return 0;
}

// Main program.
int main(int argc, char *argv[]) {
	struct sCommandLineOption *option_speed;
	struct sCommandLineOption *option_deviation_max;
	struct sCommandLineOption *option_interval_us;
	struct sMotionRandom motion_random;

	// Command line parsing
	commandline_init();
	option_speed=commandline_option_register("-s", "--speed", cCommandLine_Option_Value);
	option_deviation_max=commandline_option_register("-d", "--deviation", cCommandLine_Option_Value);
	option_interval_us=commandline_option_register("-i", "--interval-us", cCommandLine_Option_Value);
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();
	motion_random_init(&motion_random);

	// Read arguments
	if (option_interval_us->value) {
		motion_random.interval_us = atoi(option_interval_us->value);
	}
	if (option_speed->value) {
		motion_random.speed = atoi(option_speed->value);
	}
	if (option_deviation_max->value) {
		motion_random.deviation_max = atoi(option_deviation_max->value);
	}
	if (commandline_option_provided("-q", "--quiet")) {
		verbosity = 0;
	}
	if (commandline_option_provided("-v", "--verbose")) {
		verbosity = 2;
	}

	// Move randomly
	motion_random_run(&motion_random);
	return 0;
}

