/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include <khepera3.h>
#include "commandline.h"
#include "measurement.h"

// Buffers
struct sMeasurementMotor {
	int current_position;
	int current_speed;
	int current_torque;
};

struct sMeasurement {
	int sample_number;
	struct sMeasurementMotor motor_left;
	struct sMeasurementMotor motor_right;
};

#define LOG_SIZE 1024
struct sMeasurement log[LOG_SIZE];
unsigned int sample_number = 0;

// Prints the help text.
void help() {
	printf("Returns the current encoder values of both motors.\r\n");
	printf("Usage:\r\n");
	printf("  motor_getposition             Measures one set of values\r\n");
	printf("  motor_getposition -r [N]      Measures N sets of values (N=infinity if not provided)\r\n");
	printf("  motor_getposition -r N -f     Measures N sets of values and keeps them in memory before printing (for fast acquisition)\r\n");
	printf("  motor_getposition -r N -F     Measures N sets of values but does not print them\r\n");
	printf("  motor_getposition -w USEC     Waits USEC usec after each measurement\r\n");
	printf("Output format:\r\n");
	printf("  POS1 POS2 SPEED1 SPEED2 CURRENT1 CURRENT2\r\n");
}

// Takes one measurement
void measurement_take(int i) {
	log[i].sample_number = sample_number;
	khepera3_drive_get_current_position();
	khepera3_drive_get_current_speed();
	khepera3_drive_get_current_torque();
	log[i].motor_left.current_position = khepera3.motor_left.current_position;
	log[i].motor_right.current_position = khepera3.motor_right.current_position;
	log[i].motor_left.current_speed = khepera3.motor_left.current_speed;
	log[i].motor_right.current_speed = khepera3.motor_right.current_speed;
	log[i].motor_left.current_torque = khepera3.motor_left.current_torque;
	log[i].motor_right.current_torque = khepera3.motor_right.current_torque;
	sample_number++;
}

// Prints one measurement
void measurement_print(int i) {
	printf("%d %d %d %d %d %d\r\n", log[i].motor_left.current_position, log[i].motor_right.current_position, log[i].motor_left.current_speed, log[i].motor_right.current_speed, log[i].motor_left.current_torque, log[i].motor_right.current_torque);
}

// Main program.
int main(int argc, char *argv[]) {
	// Command line parsing
	commandline_init();
	measurement_init();
	measurement_commandline_prepare();
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Take continuous measures
	measurement_configuration.log_size = LOG_SIZE;
	measurement_configuration.hook_measure = &measurement_take;
	measurement_configuration.hook_print = &measurement_print;
	measurement_start();

	return 0;
}
