/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include <khepera3.h>
#include <commandline.h>

// Prints the help text.
void help() {
	printf("Runs the motors to a certain encoder position.\r\n");
	printf("Usage:\r\n");
	printf("  motor_gotoposition [OPTIONS] POS         Runs both motors to POS\r\n");
	printf("  motor_gotoposition [OPTIONS] POS1 POS2   Runs the left motor until POS1 and right motor to POS2\r\n");
	printf("  motor_gotoposition [OPTIONS] -l POS      Runs the left motor to POS\r\n");
	printf("  motor_gotoposition [OPTIONS] -r POS      Runs the right motor to POS\r\n");
	printf("Options:\r\n");
	printf("  -p     Uses a trapezoidal speed profile\r\n");
}

// Main program.
int main(int argc, char *argv[]) {
	struct sCommandLineOption *option_left;
	struct sCommandLineOption *option_right;

	// Command line parsing
	commandline_init();
	option_left = commandline_option_register("-l", "--left", cCommandLine_Option_Value);
	option_right = commandline_option_register("-r", "--right", cCommandLine_Option_Value);
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Put the wheels in normal (control) mode
	khepera3_drive_start();

	// Set the speed
	if (commandline_argument_count() == 1) {
		if (commandline_option_provided("-p", "--profile")) {
			khepera3_drive_goto_position_using_profile(atoi(commandline_argument(0)), atoi(commandline_argument(0)));
		} else {
			khepera3_drive_goto_position(atoi(commandline_argument(0)), atoi(commandline_argument(0)));
		}
	} else if (commandline_argument_count() == 2) {
		if (commandline_option_provided("-p", "--profile")) {
			khepera3_drive_goto_position_using_profile(atoi(commandline_argument(0)), atoi(commandline_argument(1)));
		} else {
			khepera3_drive_goto_position(atoi(commandline_argument(0)), atoi(commandline_argument(1)));
		}
	} else {
		if (option_left->value) {
			if (commandline_option_provided("-p", "--profile")) {
				khepera3_motor_goto_position_using_profile(&(khepera3.motor_left), atoi(option_left->value));
			} else {
				khepera3_motor_goto_position(&(khepera3.motor_left), atoi(option_left->value));
			}
		}

		if (option_right->value) {
			if (commandline_option_provided("-p", "--profile")) {
				khepera3_motor_goto_position_using_profile(&(khepera3.motor_right), atoi(option_right->value));
			} else {
				khepera3_motor_goto_position(&(khepera3.motor_right), atoi(option_right->value));
			}
		}
	}
}

