/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include <khepera3.h>
#include <commandline.h>

// Prints the help text.
void help() {
	printf("Sets the motor encoder values.\n");
	printf("Usage:\r\n");
	printf("  motor_setposition [OPTIONS] POS         Sets both motors to POS\r\n");
	printf("  motor_setposition [OPTIONS] POS1 POS2   Sets the left motor to POS1 and the right motor to POS2\r\n");
	printf("  motor_setposition [OPTIONS] -l POS      Sets the left motor to POS\r\n");
	printf("  motor_setposition [OPTIONS] -r POS      Sets the right motor to POS\r\n");
	printf("Options: none\r\n");
}

// Main program.
int main(int argc, char *argv[]) {
	struct sCommandLineOption *option_left;
	struct sCommandLineOption *option_right;

	// Command line parsing
	commandline_init();
	option_left = commandline_option_register("-l", "--left", cCommandLine_Option_Value);
	option_right = commandline_option_register("-r", "--right", cCommandLine_Option_Value);
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Set the position
	if (commandline_argument_count() == 1) {
		khepera3_drive_set_current_position(atoi(commandline_argument(0)), atoi(commandline_argument(0)));
	} else if (commandline_argument_count() == 2) {
		khepera3_drive_set_current_position(atoi(commandline_argument(0)), atoi(commandline_argument(1)));
	} else {
		if (option_left->value) {
			khepera3_motor_set_current_position(&(khepera3.motor_left), atoi(option_left->value));
		}

		if (option_right->value) {
			khepera3_motor_set_current_position(&(khepera3.motor_right), atoi(option_right->value));
		}
	}
}
