/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include <khepera3.h>
#include <commandline.h>

// Prints the help text.
void help() {
	printf("Sets the motor speed.\r\n");
	printf("Usage:\r\n");
	printf("  motor_setspeed [OPTIONS] SPEED          Sets the speed of both motors\r\n");
	printf("  motor_setspeed [OPTIONS] SPEED1 SPEED2  Sets an individual speed for both motors\r\n");
	printf("  motor_setspeed [OPTIONS] -l SPEED       Sets the speed of the left motor\r\n");
	printf("  motor_setspeed [OPTIONS] -r SPEED       Sets the speed of the right motor\r\n");
	printf("Options:\r\n");
	printf("  -p     Uses a trapezoidal speed profile\r\n");
	exit(1);
}

// Main program.
int main(int argc, char *argv[]) {
	struct sCommandLineOption *option_left;
	struct sCommandLineOption *option_right;
	int speed_left;
	int speed_right;

	// Command line parsing
	commandline_init();
	option_left = commandline_option_register("-l", "--left", cCommandLine_Option_Value);
	option_right = commandline_option_register("-r", "--right", cCommandLine_Option_Value);
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Put the wheels in normal (control) mode
	khepera3_drive_start();

	// Set the speed
	if (commandline_argument_count() == 1) {
		if (commandline_option_provided("-p", "--profile")) {
			khepera3_drive_set_speed_using_profile(atoi(commandline_argument(0)), atoi(commandline_argument(0)));
		} else {
			khepera3_drive_set_speed(atoi(commandline_argument(0)), atoi(commandline_argument(0)));
		}
	} else if (commandline_argument_count() == 2) {
		if (commandline_option_provided("-p", "--profile")) {
			khepera3_drive_set_speed_using_profile(atoi(commandline_argument(0)), atoi(commandline_argument(1)));
		} else {
			khepera3_drive_set_speed(atoi(commandline_argument(0)), atoi(commandline_argument(1)));
		}
	} else {
		if (option_left->value) {
			if (commandline_option_provided("-p", "--profile")) {
				khepera3_motor_set_speed_using_profile(&(khepera3.motor_left), atoi(option_left->value));
			} else {
				khepera3_motor_set_speed(&(khepera3.motor_left), atoi(option_left->value));
			}
		}

		if (option_right->value) {
			if (commandline_option_provided("-p", "--profile")) {
				khepera3_motor_set_speed_using_profile(&(khepera3.motor_right), atoi(option_right->value));
			} else {
				khepera3_motor_set_speed(&(khepera3.motor_right), atoi(option_right->value));
			}
		}
	}
}
