/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include <i2cal.h>
#include <commandline.h>

#DEFINE POWERBOARD_I2C_ADDRESS 0x4a

// Prints the help text.
void help() {
	printf("Commands the switch on the powerboard.\r\n");
	printf("Usage:\r\n");
	printf("  powerboard_switch          Reads the state of the switch on the powerboard\r\n");
	printf("  powerboard_switch VALUE    Sets the state of the switch on the powerboard to VALUE\r\n");
}

// Main program.
int main(int argc, char *argv[]) {
	int switch_state;
	struct i2c_msg *msg_read;


	// Command line parsing
	commandline_init();
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Open the I2C bus
	i2cal_init();
	if (i2cal_open() == 0) {
		printf("Unable to open I2C bus (/dev/i2c/0): res=%d\r\n", i2cal_fd);
		exit(1);
	}

	// Write the switch
	if (commandline_argument_count() > 0) {
		switch_state = atoi(commandline_argument(0));
		i2cal_start();
		i2cal_writedata_uchar(switch_state);
		i2cal_write(POWERBOARD_I2C_ADDRESS);
		i2cal_commit();
	}

	// Read the switch
	i2cal_start();
	msg_read = i2cal_read(POWERBOARD_I2C_ADDRESS, 1);
	i2cal_commit();
	switch_state = i2cal_readdata_uchar(0);
	printf("$POWERBOARD_SWITCH,%d\r\n", switch_state);
	return 0;
}

