/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include "khepera3.h"
#include "commandline.h"
#include "tcp_nmea.h"

// Prints the help text.
void help() {
	printf("Opens a TCP port for listening and executes motor commands of a connected TCP client.\r\n");
	printf("Usage:\r\n");
	printf("  tcp_server             Listens on TCP port 3000\r\n");
	printf("  tcp_server -p PORT     Listens on port PORT\r\n");
}

// This method is called each time a NMEA message is received.
void tcp_server_nmea_receive_hook(struct sNMEAMessage *m, int withchecksum) {
	int i;

	if (strcmp(m->command, "SPEED") == 0) {	// e.g. $SPEED,1000,1000
		// Set the speed according to the arguments
		if (m->argument_count >= 2) {
			khepera3_drive_set_speed(atoi(m->argument[0]), atoi(m->argument[1]));
		}
	} else if (strcmp(m->command, "IR") == 0) {	// $IR
		// Report the IR proximity values
		khepera3_infrared_proximity();
		fprintf(tcp_nmea_connection_filehandle, "$IR");
		for (i = 0; i < 11; i++) {
			fprintf(tcp_nmea_connection_filehandle, ",%d", khepera3.infrared_proximity.sensor[i]);
		}
		fprintf(tcp_nmea_connection_filehandle, "\r\n");
	}
}

// Executes the main loop.
void tcp_server_run(int port) {
	// Set up the listener socket
	tcp_nmea_init(&tcp_server_nmea_receive_hook);
	tcp_nmea_start(port);

	// Put the wheels in normal (control) mode
	khepera3_drive_start();

	while (1) {
		// Check if new data from is available on the TCP port and process it (this will call tcp_server_nmea_receive_hook)
		tcp_nmea_receive();

		// Sleep for a while
		usleep(20000);
	}
}

// Main program.
int main(int argc, char *argv[]) {
	struct sCommandLineOption *option_port;
	int port = 3000;

	// Command line parsing
	commandline_init();
	option_port = commandline_option_register("-p", "--port", cCommandLine_Option_Value);
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Read arguments
	if (option_port->value) {
		port = atoi(option_port->value);
	}

	// Run server
	tcp_server_run(port);
	return 0;
}

