/*!
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include "khepera3.h"
#include "commandline.h"
#include "trackposition.h"

// The waiting time in the main loop
int wait_us;

// Prints the help text.
void help() {
	printf("Tracks the robot position using odometry information.\r\n");
	printf("Usage: trackposition [OPTIONS]\r\n");
	printf("  -w --wait-us US     Waits US us the main loop (default: 0 ms, max speed)\r\n");
}

// Tracks the position.
int trackposition() {
	int res;
	struct sTrackPosition tr;

	res=trackposition_start(&tr);
	if (res==0) {
		printf("WARNING: Odometry configuration file (/etc/khepera/odometry) not found. Using default values.\n");
	}
	while (1) {
		trackposition_step(&tr);
		printf("$POS,%f,%f,%f,%d,%d\r\n", tr.x, tr.y, tr.theta, tr.pos_left_prev, tr.pos_right_prev);
		
		if (wait_us) {
			usleep(wait_us);
		}
	}

	return 0;
}

// Main program.
int main(int argc, char *argv[]) {
	struct sCommandLineOption *option_wait_us;

	// Command line parsing
	commandline_init();
	option_wait_us = commandline_option_register("-w", "--wait-us", cCommandLine_Option_Value);
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Read arguments
	wait_us=0;
	if (option_wait_us->value) {
		wait_us = atoi(option_wait_us->value);
	}

	// Run server
	trackposition();
	return 0;
}
