/*! 
 * (c) 2006-2007 EPFL, Lausanne, Switzerland
 * Thomas Lochmatter
 */

#include <khepera3.h>
#include <commandline.h>

// Prints the help text.
void help() {
	printf("Enabled or disables the ultrasound sensors.\r\n");
	printf("Usage:\r\n");
	printf("  ultrasound_enable [left|0] [frontleft|1] [front|2] [frontright|3] [right|4]\r\n");
	printf("  ultrasound_enable [--mask 0-31]\r\n");
}

// Main program.
int main(int argc, char *argv[]) {
	int i;
	int mask=0;
	int sensor_number;
	const int sensor[5]={1, 2, 4, 8, 16};
	struct sCommandLineOption *option_mask;

	// Command line parsing
	commandline_init();
	option_mask=commandline_option_register("-m", "--mask", cCommandLine_Option_Value);
	commandline_parse(argc, argv);

	// Help
	if (commandline_option_provided("-h", "--help")) {
		help();
		exit(1);
	}

	// Initialization
	khepera3_init();

	// Read command line (--mask option and arguments)
	if (option_mask->value) {
		mask=atoi(option_mask->value);
	}
	for (i=0; i<commandline_argument_count(); i++) {
		sensor_number=khepera3_ultrasound_getsensorbyname(commandline_argument(0), -1);
		mask |= sensor[sensor_number];
	}

	// Enable ultrasound sensors
	khepera3_ultrasound_enable(mask);

	// Print report
	for (i=0; i<5; i++) {
		if (mask & sensor[i]) {
			printf("+ ");
		} else {
			printf("- ");
		}
		printf("%s\r\n", cKhepera3SensorsUltrasound[i]);
	}

	return 0;
}
